###########################################################
#
# Copyright (c) 2010, Southpaw Technology
#                     All Rights Reserved
#
# PROPRIETARY INFORMATION.  This software is proprietary to
# Southpaw Technology, and is not to be reproduced, transmitted,
# or disclosed in any way without written permission.
#
#

__all__ = ['PluginWdg', 'PluginEditWdg', 'PluginUploadRegisterWdg', 'PluginUploadRegisterCbk', 'PluginInstallIntoProjectCbk']

from pyasm.common import Environment, TacticException, Config
from pyasm.command import Command
from pyasm.web import DivWdg, Table
from pyasm.widget import ButtonWdg, ProdIconButtonWdg, TextWdg, TextAreaWdg, CheckboxWdg
from pyasm.search import Search

import os, codecs
import zipfile, shutil

from tactic.ui.common import BaseRefreshWdg
from tactic.ui.container import ResizableTableWdg
from tactic.ui.widget import ActionButtonWdg


class PluginWdg(BaseRefreshWdg):

    def get_display(my):

        div = DivWdg()
        div.add_class("spt_plugin_top")
        my.set_as_panel(div)

        div.add_color("background", "background")

        upload_div = DivWdg()
        upload_div.add_style("padding: 10px")
        upload_div.add_style("width: 600px")



        # add the main layout
        table = ResizableTableWdg()
        table.add_color("color", "color")
        div.add(table)

        table.add_row()
        left = table.add_cell()
        left.add_border()
        left.add_style("width: 250px")
        left.add_style("height: 400px")

        left.add(my.get_plugins_wdg() )

        right = table.add_cell()
        right.add_border()
        right.add_style("width: 400px")
        right.add_style("height: 400px")
        right.add_style("padding: 5px")


        edit = PluginEditWdg()
        right.add(edit)

        return div



    def get_plugins_wdg(my):
        div = DivWdg()

        search_type = 'sthpw/plugin'
        search = Search(search_type)
        plugins = search.get_sobjects()


        search_type = 'config/plugin'
        search = Search(search_type)
        active_plugins = search.get_sobjects()


        title_div = DivWdg()
        div.add(title_div)
        title_div.add("Plugin List")
        title_div.add_style("font-size: 14px")
        title_div.add_style("font-weight: bold")
        title_div.add_gradient("background", "background", 0, -10)
        title_div.add_style("padding: 5px")

        new_button = ActionButtonWdg(title='New', size='medium', tip='Create a new plugin')
        title_div.add(new_button)
        new_button.add_style("margin-top: -22px")
        new_button.add_style("margin-right: -5px")
        new_button.add_style("float: right")
        new_button.add_behavior( {
        'type': 'click_up',
        'cbjs_action': '''
        var top = bvr.src_el.getParent(".spt_plugin_top");
        var edit = top.getElement(".spt_plugin_edit");
        var class_name = "tactic.ui.app.PluginEditWdg";
        var kwargs = {
            mode: 'insert'
        };
        spt.panel.load(edit, class_name, kwargs);
        '''
        } )




        add_button = ActionButtonWdg(title='+', size='small', tip='Install a plugin')
        title_div.add(add_button)
        add_button.add_style("margin-top: -22px")
        add_button.add_style("margin-right: -5px")
        add_button.add_style("float: right")

        add_button.add_behavior( {
        'type': 'click_up',
        'cbjs_action': '''
        var top = bvr.src_el.getParent(".spt_plugin_top");
        var edit = top.getElement(".spt_plugin_edit");
        var class_name = "tactic.ui.app.PluginUploadRegisterWdg";
        var kwargs = {
          search_key: bvr.search_key
        };
        spt.panel.load(edit, class_name, kwargs);
        '''
        } )


        div.add("<br/>List of available plugins")
 

        for plugin in plugins:
            plugin_div = DivWdg()
            div.add(plugin_div)
            plugin_div.add_style("padding: 5px")
            plugin_div.add_class("hand")

            description = plugin.get_value("description")
            code = plugin.get_code()

            plugin_div.add(code)

            search_key = plugin.get_search_key()

            plugin_div.add_behavior( {
            'type': 'click_up',
            'search_key': search_key,
            'cbjs_action': '''
            var top = bvr.src_el.getParent(".spt_plugin_top");
            var edit = top.getElement(".spt_plugin_edit");
            var class_name = "tactic.ui.app.PluginEditWdg";
            var kwargs = {
              search_key: bvr.search_key
            };
            spt.panel.load(edit, class_name, kwargs);
            '''
            } )

            hover = plugin_div.get_color("background", -5)
            plugin_div.add_behavior( {
            'type': 'hover',
            'hover': hover,
            'cbjs_action_over': '''
            bvr.src_el.setStyle("background", bvr.hover);
            ''',
            'cbjs_action_out': '''
            bvr.src_el.setStyle("background", '');
            '''
            } )

            checkbox = CheckboxWdg("loaded")
            plugin_div.add(checkbox)

            for xx in active_plugins:
                if xx.get_code() == plugin.get_code():
                    checkbox.set_checked()
                    break







        return div


class PluginEditWdg(BaseRefreshWdg):
    def get_display(my):

        top = my.top
        my.set_as_panel(top)
        top.add_class("spt_plugin_edit")


        title_wdg = DivWdg()
        top.add(title_wdg)
        title_wdg.add_style("font-size: 14px")
        title_wdg.add_style("font-weight: bold")
        title_wdg.add_style("margin-bottom: 10px")

        my.mode = my.kwargs.get("mode")
        if my.mode != 'insert':

            search_key = my.kwargs.get("search_key")
            if not search_key:
                top.add("No Plugin Selected")
                return top

            plugin = Search.get_by_search_key(search_key)
            code = plugin.get_code()
            description = plugin.get_value("description")
            version = plugin.get_value("version")
            manifest = plugin.get_value("manifest")

            title_wdg.add("Edit Plugin Info")

        else:
            code = ''
            description = ''
            version = ''
            manifest = ''

            plugin = None

            title_wdg.add("Create New Plugin")

        table = Table()
        top.add(table)
        table.add_color("color", "color")



        table.add_row()
        td = table.add_cell()
        td.add("Code: ")
        td.add_style("vertical-align: top")
        text = TextWdg("code")
        td = table.add_cell()
        td.add(text)
        text.set_value(code)




        table.add_row()
        td = table.add_cell()
        td.add("Description: ")
        td.add_style("vertical-align: top")
        text = TextAreaWdg("description")
        td = table.add_cell()
        td.add(text)
        text.set_value(description)


        table.add_row()
        td = table.add_cell()
        td.add("Version: ")
        td.add_style("vertical-align: top")
        text = TextWdg("version")
        td = table.add_cell()
        td.add(text)
        text.set_value(version)


        table.add_row()
        td = table.add_cell()
        td.add("Manifest: ")
        td.add_style("vertical-align: top")
        text = TextAreaWdg("manifest")
        td = table.add_cell()
        td.add(text)
        text.set_value(manifest)


        top.add("<hr/>")

        if plugin:
            search = Search("config/plugin")
            search.add_filter("code", code)
            installed = search.get_sobject()
        else:
            installed = False

        if not installed:

            button = ActionButtonWdg(title='Install', tip='Install Plugin into Current Project')
            top.add(button)
            button.add_behavior( {
            'type': 'click_up', 
            'cbjs_action': '''
            spt.api.app_busy_show("Installing Plugin");

            var top = bvr.src_el.getParent(".spt_plugin_edit");
            var search_key = top.getAttribute("spt_search_key");

            var class_name = 'tactic.command.PluginInstaller';
            var kwargs = {
                search_key: search_key,
                mode: 'install',
            };

            var server = TacticServerStub.get();
            server.execute_cmd(class_name, kwargs);

            spt.panel.refresh(top);

            spt.api.app_busy_hide();

            '''
            })

        else:

            button = ActionButtonWdg(title='Remove', tip='Remove Plugin from current preject')
            top.add(button)
            button.add_behavior( {
            'type': 'click_up', 
            'cbjs_action': '''
            spt.api.app_busy_show("Creating New Plugin", "");

            var top = bvr.src_el.getParent(".spt_plugin_edit");
            var search_key = top.getAttribute("spt_search_key");

            var class_name = 'tactic.command.PluginUninstaller';
            var kwargs = {
                search_key: search_key,
            };

            var server = TacticServerStub.get();
            server.execute_cmd(class_name, kwargs);

            spt.panel.refresh(top);

            spt.api.app_busy_hide();

            '''
            })




        return top



class PluginUploadRegisterWdg(BaseRefreshWdg):

    def get_display(my):

        top = my.top

        top.add("Plugin File: ")

        file_div = DivWdg()
        file_div.add_class("spt_plugin_file")
        top.add(file_div)
        file_div.add_style("padding: 10px")
        file_div.add("-- None Selected --")


        upload_button = ActionButtonWdg(title="Browse")
        #upload_button = ProdIconButtonWdg("Browse")
        upload_button.add_style("float: left")
        upload_button.add_behavior( {
            'type': 'click_up',
            'cbjs_action': '''
            spt.api.app_busy_show("Selecting files", "");
            var applet = spt.Applet.get();
            var files = applet.open_file_browser()
            if (files.length < 0) {
                spt.api.app_busy_hide();
                alert("Please select a TACTIC plugin file");
                return;
            }

            var applet = spt.Applet.get();
            var server = TacticServerStub.get();

            // take the file and upload it
            for (var i = 0; i < files.length; i++) {
                spt.api.app_busy_show("Installing plugin", files[i]);

                var file = files[i];
                file = file.replace(/\\\\/g, "/");
                var parts = file.split("/")
                file = parts[parts.length-1];
                var parts = file.split("_template");
                var project_code = parts[0];

                applet.upload_file(files[i]);

                // for now, only support projects templates
                var class_name = 'tactic.command.ProjectTemplateInstallerCmd';
                var kwargs = {
                    'project_code': project_code,
                    'path': files[i]
                }
                server.execute_cmd(class_name, kwargs);



            }

            //var top = bvr.src_el.getParent(".spt_plugin_top");
            //var file = top.getElement(".spt_plugin_file");
            //file.innerHTML = files[0];



            spt.api.app_busy_hide();
            '''
        } )
        top.add(upload_button)



        install_button = ActionButtonWdg(title="Register")
        install_button.add_behavior( {
            'type': 'click_up',
            'cbjs_action': '''
                var top = bvr.src_el.getParent(".spt_plugin_top");
                var file = top.getElement(".spt_plugin_file");
                var path = file.innerHTML;

                var server = TacticServerStub.get();
                spt.api.app_busy_show("Uploading ...", "File: " + path);
                server.upload_file(path)

                var kwargs = {
                    'path': path
                };
                spt.api.app_busy_show("Installing ...", "File: " + path);
                server.execute_cmd("tactic.ui.app.PluginUploadRegisterCbk", kwargs);

                spt.api.app_busy_show("Refreshing SideBar ...", "");
                spt.panel.refresh("side_bar");
                spt.panel.refresh(top);

                spt.api.app_busy_show("Success", "");
                spt.api.app_busy_hide();

            ''',
        } )


        top.add(install_button)

        plugins_div = DivWdg()
        top.add(plugins_div)
        plugins_div.add_style("padding: 10px")

        title_div = DivWdg()
        title_div.add("Currently Defined Plugins")
        plugins_div.add(title_div)
        title_div.add_style("font-size: 14px")
        title_div.add_gradient("background", "background", 0, -10)
        title_div.add_style("padding", "10px")
        title_div.add_style("font-weight", "bold")
        title_div.add_border()

        from tactic.ui.panel import TableLayoutWdg
        #table = TableLayoutWdg(search_type='config/plugin',view='table')
        table = TableLayoutWdg(search_type='sthpw/plugin',view='table', mode='simple')
        plugins_div.add(table)


        return top


class PluginUploadRegisterCbk(Command):

    def execute(my):

        my.base_dir = Environment.get_plugin_dir()
        upload_dir = Environment.get_upload_dir()

        orig_path = my.kwargs.get("path")
        orig_path = orig_path.replace("\\", "/")

        basename = os.path.basename(orig_path)
        plugin_name, ext = os.path.splitext(basename)
        my.plugin_dir = "%s/%s" % (my.base_dir, plugin_name)

        upload_path = "%s/%s" % (upload_dir, basename)


        if not os.path.exists(upload_path):
            raise TacticException("File [%s] does not exist" % upload_path)

        if not basename.endswith(".zip"):
            raise TacticException("Plugin file requires zip extension")


        plugin_dir = "%s/%s" % (my.base_dir, plugin_name)

        # remove the plugin dir
        if os.path.exists(plugin_dir):
            shutil.rmtree(plugin_dir)
            os.makedirs(plugin_dir)


        # extract the zip file
        zip = zipfile.ZipFile(upload_path)
        info_list = zip.infolist()
        info_list.sort()

        for info in info_list:
            out_path = "%s/%s" % (my.base_dir, info.filename)

            # check if this is a directory
            if out_path.endswith("/"):
                # eliminate if it already exists
                if os.path.exists(out_path):
                    shutil.rmtree(out_path)
                os.makedirs(out_path)


        for info in info_list:
            out_path = "%s/%s" % (my.base_dir, info.filename)

            # check if this is a directory
            if out_path.endswith("/"):
                continue

            # finally make sure that the output directory exists
            out_dir = os.path.dirname(out_path)
            if not os.path.exists(out_dir):
                os.makedirs(out_dir)
            

            print "Extracting: ", out_path

            #out_dirname = os.path.dirname(out_path)
            #if not os.path.exists(out_dirname):
            #    os.makedirs(out_dirname)

            fout = codecs.open(out_path, 'w')
            buffer = zip.read(info.filename)
            fout.write(buffer)
            fout.close()

        # load the plugin info into the plugin table
        from tactic.command import PluginInstaller
        installer = PluginInstaller(mode='register', code=plugin_name)
        installer.execute()

 




class PluginInstallIntoProjectCbk(Command):
    '''This actually installs the plugin in to the project'''
    def execute(my):

        # get the plugin directory
        my.plugin_dir = "%s/%s" % (my.base_dir, plugin_name)

        # get the manifest path for the plugin
        manifest_path = "%s/manifest.xml" % plugin_dir
        if not os.path.exists(upload_path):
            raise TacticException("Plugin has no manifest file [%s]" % manifest_path)

        # read the manifest file
        f = codecs.open(manifest_path, 'r')
        manifest = f.read()
        f.close()


        # install the plugin into the project
        from tactic.command import PluginInstaller
        installer = PluginInstaller(manifest=manifest)
        installer.execute()

        



