###########################################################
#
# Copyright (c) 2005-2012, Southpaw Technology
#                     All Rights Reserved
#
# PROPRIETARY INFORMATION.  This software is proprietary to
# Southpaw Technology, and is not to be reproduced, transmitted,
# or disclosed in any way without written permission.
#
#
#


__all__ = ['Html5UploadWdg', 'TestHtml5UploadWdg']

from tactic.ui.common import BaseRefreshWdg

from pyasm.web import HtmlElement, DivWdg


class Html5UploadWdg(BaseRefreshWdg):

    def init(my):

        name = my.kwargs.get("name")
        if not name:
            name = "table_upload"

        my.form = HtmlElement.form()
        my.form_id = my.form.set_unique_id(name)

    def get_upload_id(my):
        return my.form_id

    def get_display(my):

        top = my.top
        form = my.form
        form.add_style("margin: 0px")
        form.add_style("padding: 0px")


        top.add(form)

        input = HtmlElement.input()
        form.add(input)
        input.set_attr("name", "file")
        input.add_class("spt_file")
        input.set_attr("type", "file")
        #input.add_style("display: none")
        #input.add_style("visibility: hidden")
        input.add_style("position: absolute")
        input.add_style("margin-left: -1000px")


        form.add_behavior( {
            'type': 'load',
            'cbjs_action': '''
spt.html5upload = {};
spt.html5upload.form = null;
spt.html5upload.files = [];
spt.html5upload.events = {};


spt.html5upload.set_form = function(form) {
    if (!form) {
        spt.alert('Cannot initialize the HTML upload. Invalid form detected.');
        return;
    }
    spt.html5upload.form = form;
}

// get the last one in the list since this FileList is readonly and it is additive if multiple attr is on
spt.html5upload.get_file = function() {
    var files = spt.html5upload.get_files();
    if (files.length == 0) {
        return null;
    }
    return files[files.length-1];
}


//FIXME: it doesn't need to be stored as files since it should be called
// every time an upload occurs
spt.html5upload.get_files = function() {
    var file_input = spt.html5upload.form.getElement('.spt_file');
    spt.html5upload.files = file_input.files;
    return spt.html5upload.files;
}


spt.html5upload.select_file = function(onchange) {
    var files = spt.html5upload.select_files(onchange);
    if (!files) {
        spt.alert('You may need to refresh this page.');
        return null;
    }
        
    if (files.length == 0) {
        return null;
    }
    return files[0];
}


spt.html5upload.select_files = function(onchange) {
    var form = spt.html5upload.form;
    if (!form) {
        spt.alert('Cannot locate the upload form. Refresh this page/tab and try again');
        return; 
    }

    var el = form.getElement(".spt_file") ;
  
    // ensure this listener is only added once
    if (!spt.html5upload.events['select_file']){
        el.addEventListener("change", onchange, true);
    }

    // This is necessary for Qt on a Mac??
    if (spt.browser.is_Qt() || spt.browser.is_Safari()) {
        setTimeout( function() {
            el.click();
            spt.html5upload.events['select_file'] = onchange;
        }, 100 );
    }
    else {
        el.click();
        spt.html5upload.events['select_file'] = onchange;
    }

    // FIXME: this is not very useful as the select file is async, but
    // is required for later code not to opne a popup
    spt.html5upload.files = el.files;
    return spt.html5upload.files;
}

// clears the otherwise readonly filelist, disables the input
spt.html5upload.clear = function() {
    var form = spt.html5upload.form;
    var el = form.getElement(".spt_file");
    el.value = null;
    
    // cloning clears all the event listeners
    var new_element = el.cloneNode(true);
    el.parentNode.replaceChild(new_element, el);

    spt.html5upload.events['select_file'] = null;
    
}


spt.html5upload.upload_failed = function(evt) {
    spt.app_busy.hide();
    spt.alert("Upload failed");
}


spt.html5upload.upload_file = function(kwargs) {

  if (!kwargs) {
      kwargs = {};
  }

  var xhr = new XMLHttpRequest();
  var form = spt.html5upload.form;
  var el = form.getElement(".spt_file") ;
  var file_name = el.value;
  if (file_name.test(/,/)) {
      spt.alert('Comma , is not allowed in file name.');
      spt.html5upload.clear();
      return;
  }

  var upload_complete = kwargs.upload_complete;
  var upload_progress = kwargs.upload_progress;
  var upload_failed = spt.html5upload.upload_failed;
  var login_ticket = kwargs.ticket;
    

  //var fd = form.getFormData();
  var fd = new FormData(form);
  if (login_ticket)
      fd.append('login_ticket', login_ticket);

  /* event listeners */
  if (upload_progress) {
      xhr.upload.addEventListener("progress", upload_progress, false);
  }
  if (upload_complete) {
      xhr.addEventListener("load",  upload_complete, false);
  }
  if (upload_failed) {
      xhr.addEventListener("error", upload_failed, false);
  }
  //xhr.addEventListener("abort", uploadCanceled, false);
  xhr.open("POST", "/tactic/default/UploadServer/", true);
  xhr.send(fd);

}



        '''
        } )


        return top



class TestHtml5UploadWdg(BaseRefreshWdg):

    def get_display(my):

        top = my.top

        upload = Html5UploadWdg(name="formxyz")
        top.add(upload)
        upload_id = upload.get_upload_id()

        from tactic.ui.widget import ActionButtonWdg
        button = ActionButtonWdg(title="Upload")
        top.add(button)
        button.add_behavior( {
            'type': 'click_up',
            'upload_id': upload_id,
            'cbjs_action': '''

            // set the form
            spt.html5upload.form = $(bvr.upload_id);

            // set an action for completion
            var upload_complete = function(evt) {
                var search_key = "sthpw/login?code=admin";
                var server = TacticServerStub.get();
                var file = spt.html5upload.get_file();
                if (file) {
                   file_name = file.name;
                     
                   server.simple_checkin(search_key, "icon", file_name, {mode:'uploaded'});
                }
                else 
                  alert('Error: file object cannot be found.')

            }

            var percent_complete = function(evt) {
                var percent = Math.round(evt.loaded * 100 / evt.total);
            }

	    var onchange = function () {
                spt.html5upload.upload_file( {
                  upload_complete: upload_complete,
                  upload_progress: percent_complete 
                } );
	    }


	
            spt.html5upload.select_file( onchange);

            '''
        } )

        return top





