###########################################################
#
# Copyright (c) 2005-2008, Southpaw Technology
#                     All Rights Reserved
#
# PROPRIETARY INFORMATION.  This software is proprietary to
# Southpaw Technology, and is not to be reproduced, transmitted,
# or disclosed in any way without written permission.
#
#
#

# This module contains a collection of generalized utility widgets

__all__ = ['CalendarWdg', 'CalendarMonthWdg', 'CalendarTimeWdg', 'CalendarInputWdg', 'TimeInputWdg', 'CalendarTimeInputWdg']

from pyasm.common import Date, Common, Config
from pyasm.biz import NamingUtil
from pyasm.web import Widget, Table, DivWdg, SpanWdg, WebContainer, FloatDivWdg
from pyasm.widget import IconWdg, IconButtonWdg, TextWdg, HiddenWdg, BaseInputWdg, SelectWdg, ProdIconButtonWdg
from tactic.ui.common import BaseRefreshWdg

from datetime import datetime
from dateutil import parser

try:
    from calendar import Calendar
    HAS_CALENDAR = True
except ImportError, e:
    HAS_CALENDAR = False 
    
import calendar



class CalendarWdg(BaseRefreshWdg):

    WEEKDAYS = ['Monday', 'Tuesday', 'Wednesday', 'Thursday', 'Friday','Saturday','Sunday']

    MONTHS = ['January','Febuary','March','April','May','June','July','August','September','October','November','December']

    ARGS_KEYS = {
        'year': 'The starting year of the calendar',
        'month': 'The starting month of the calendar',
        'size': 'full|abbr|letter - determines the # of letters displayed on the month',
        'is_refresh': 'true|false - determines whether to draw the top node or not',
        'day_cbk': 'callback when a day is clicked',
        'name': 'name of the hidden input that is set',
        'search_key' : 'search_key for sobject that contains this date',
        'tbody_id':  'tbody_id',
        'col_name': 'col_name for this date entry',

        'first_day_of_week': 'integer indicating first day of the week',
        'layout_version': '1 or 2'
       
    }

    def init(my):
        my.top_wdg = DivWdg()
        my.top_wdg.add_style("z-index: 100")
        my.top_wdg.add_class("spt_calendar_top")
        my.top_wdg.add_style("-moz-user-select: none")

        #my.top_wdg.add_style("background: #444")
        my.top_wdg.add_color("background", "background")
        my.top_wdg.add_color("color", "color")
        my.top_wdg.add_style("padding: 3px")
        my.top_wdg.set_id('calendar')
        my.set_as_panel(my.top_wdg)

        if my.kwargs.get("show_border") not in ['false', False]:
            my.top_wdg.add_border()
            my.top_wdg.set_box_shadow("1px 1px 2px 2px")

        my.day_cbks = []
        day_cbk = my.kwargs.get("day_cbk")
        if day_cbk:
            my.day_cbks.insert(0, day_cbk)



        date = Date()
        web = WebContainer.get_web()

        my.is_refresh = False
        is_refresh = web.get_form_value("is_refresh")
        if not is_refresh:
            is_refresh = my.kwargs.get('is_refresh')
        if is_refresh == "true":
            my.is_refresh = True

        # the 3 cbk_values are search_key, tbody_id, col_name
        my.search_key = web.get_form_value("search_key")
        my.top_id = web.get_form_value("top_id")
        my.col_name = web.get_form_value("col_name")
        my.date = web.get_form_value("date")
        my.year, my.month = '', ''
        if my.date:
            date = Date(my.date)
            my.year = date.get_year()
            my.month = date.get_month()
        if not my.year:    
            my.year = web.get_form_value("year")
        if not my.year:
            my.year = my.kwargs.get('year')
        if not my.year:
            my.year = date.get_year()
        
        
        if not my.month:
            my.month = web.get_form_value("month")
        if not my.month:
            my.month = my.kwargs.get('month')
        if not my.month:
            # PREV (this caused bug of cal wdg being a month in the future):  my.month = int(date.get_month())+1
            my.month = int(date.get_month())
        
        my.year = int(my.year)    
        my.month = int(my.month)

        my.weeks = my.get_weeks(my.year, my.month)



    def add_style(my, style):
        my.top_wdg.add_style(style)


    def add_day_cbk(my, cbk):
        my.day_cbks.append(cbk)


    # HACK to get it to be an input!!
    def get_option(my, name):
        return my.kwargs.get(name)

    def get_title(my):
        return my.get_name()


    def get_display(my):

        # handle size
        size_str = my.kwargs.get("size")
        if size_str == "full":
            my.size = 0
        elif size_str == 'abbr':
            my.size = 2
        elif size_str == 'letter':
            my.size = 1
        else:
            my.size = 2


        # add some boundary conditions
        if my.month < 1:
            my.month = 12
            my.year -= 1

        elif my.month > 12:
            my.month = 1
            my.year += 1

        # remember the new values
        my.kwargs['year'] = my.year
        my.kwargs['month'] = my.month

        calendar = my.get_calendar_wdg(my.year, my.month)
        if my.is_refresh:
            return calendar

        inner = DivWdg()
        my.top_wdg.add(inner)
        inner.add( calendar )


        my.top_wdg.add_relay_behavior( {
            'type': 'click',
            'bvr_match_class': 'spt_calendar_day',
            'cbjs_action': '''
spt.calendar = {}
spt.calendar.set_day = function(evt, bvr)
{
    var src_el = bvr.src_el;
    var value = src_el.getAttribute("spt_date");
    //alert(src_el.getAttribute("spt_date"));
    input = spt.get_cousin(src_el, ".spt_calendar_top", ".spt_calendar_date")
    input.value = value;

    var top = spt.get_parent(src_el, ".spt_calendar_top");
    top.setStyle("display", "none");
}
            ''' } )

        if my.kwargs.get("is_refresh") in [True, 'true']:
            return inner
        else:
            return my.top_wdg


    """#TO BE REMOVED

    def set_as_panel(my, widget):
        widget.add_class("spt_panel")

        widget.add_attr("spt_class_name", Common.get_full_class_name(my) )
        for name, value in my.kwargs.items():
            widget.add_attr("spt_%s" % name, value)
    """


    def get_weeks(my, year, month):
        #HAS_CALENDAR = False
        if HAS_CALENDAR:
            first_day_of_week = my.kwargs.get("first_day_of_week")
            if first_day_of_week or first_day_of_week == 0:
                first_day_of_week = int(first_day_of_week)
            else:
                first_day_of_week = Config.get_value("install","first_day_of_week")
                if first_day_of_week == '':
                    first_day_of_week = 6
                else:
                    first_day_of_week = int(first_day_of_week)

            my.cal = Calendar(first_day_of_week)
            weeks = my.cal.monthdatescalendar(my.year, month)
        else:
            # for Python 2.4, build it up manually
            class Day(object):
                def weekday(my):
                    if my.day == 0:
                        return 0
                    weekday = my.day - 1
                    return weekday

            weeks_old = calendar.monthcalendar(my.year, month)
            weeks = []
            for week_old in weeks_old:
                week = []
                for day_old in week_old:
                    day = Day()
                    if day_old == 0:
                        day.month = 0
                        day.day = 0
                    else:
                        day.month = month
                        day.day = day_old
                    day.year = year

                    week.append(day)
                weeks.append(week)

        return weeks


    def get_calendar_wdg(my, year, month):
        my.year = year
        my.month = month


        weeks = my.weeks

        widget = DivWdg()
        widget.add_style("width: 100%")

        
        

        # add the header widget
        if my.kwargs.get("show_header") not in ['false', False]:
            header = my.get_header_wdg()
            widget.add(header)


        # add the main table
        table = Table()
        table.add_row(css="minimal")
        table.add_color("color", "color")

        table.add_style("margin-left: auto")
        table.add_style("margin-right: auto")


        # get all of the day widgets
        day_wdgs = []

        # get the various left widgets
        left_wdgs = []
        has_left_wdgs = False
        for week in weeks:
            left_wdg = my.get_week_left_wdg(week)
            if left_wdg:
                has_left_wdgs = True
            left_wdgs.append(left_wdg)

        # get the various left widgets
        right_wdgs = []
        has_right_wdgs = False
        for week in weeks:
            right_wdg = my.get_week_right_wdg(week)
            if right_wdg:
                has_right_wdgs = True
            right_wdgs.append(right_wdg)



        # add the various cells of the month
        week = weeks[0]
        if has_left_wdgs:
            table.add_cell("&nbsp;")
        for day in week:
            td = table.add_cell()
            td.add(my.get_day_header_wdg(day) )
        if has_right_wdgs:
            table.add_cell("&nbsp;")

        border_type = my.kwargs.get("border_type")
        if not border_type:
            border_type = 'bottom'

        
        for i, week in enumerate(weeks):
            table.add_row()

            
            if has_left_wdgs:
                left_wdg = left_wdgs[i]
                if left_wdg:
                    td = table.add_cell(left_wdg)
                    td.add_style('vertical-align: top')
                else:
                    td.add_cell("&nbsp;")

            for day in week:
                td = table.add_cell()
                td.add_style("padding: 0px")
                td.add(my.get_day_wdg(month, day))
                td.add_style("vertical-align: top")

                if border_type == 'all':
                    td.add_border()
                else:
                    td.add_color("border-color", "border")
                    td.add_style("border-style", "solid")
                    td.add_style("border-width", "0px 0px 1px 0px")

            if has_right_wdgs:
                right_wdg = right_wdgs[i]
                if right_wdg:
                    td = table.add_cell(right_wdg)
                    td.add_style('vertical-align: top')
                else:
                    td.add_cell("&nbsp;")



    
        widget.add(table)
        return widget



    def get_header_wdg(my):

        month = my.month
        year = my.year

        header = Table()
        header.add_style("width: 100%")
        header.add_color("color", "color")


        # add the month navigators
        prev_month_wdg = my.get_prev_month_wdg()
        next_month_wdg = my.get_next_month_wdg()


        # add the today icon
        header.add_row()
        today_icon = my.get_today_icon()
        td = header.add_cell( today_icon )



        date_str = "%s, %s" % (my.MONTHS[month-1], my.year)
        month_wdg = DivWdg()
        month_wdg.add_color("color", "color")
        month_wdg.add(date_str)


        month_wdg.add_behavior({
            'type': 'click_up',
            'name': my.col_name,
            'year': my.year,
            'month': my.month,
            'cbjs_action': '''
            var el = bvr.src_el.getParent('.spt_calendar_top');
            var class_name = 'tactic.ui.widget.CalendarMonthWdg';
            var kwargs = {
                name: bvr.name,
                month: bvr.month,
                year: bvr.year
            };
            spt.panel.load(el, class_name, kwargs);
            '''
        })


        # add the navigator
        td = header.add_cell()
 
        month_nav = Table()
        month_nav.add_style("margin-left: auto")
        month_nav.add_style("margin-right: auto")
        td.add(month_nav)
        month_nav.add_row()
        month_nav.add_cell( prev_month_wdg )
        td = month_nav.add_cell(month_wdg)
        td.add_style("text-align: center")
        td.add_style("width: 105px")
        month_nav.add_cell( next_month_wdg)

        prev_month_wdg.add_style("float: left")
        next_month_wdg.add_style("float: right")



        # add the close icon
        close_icon = my.get_close_icon()
        td = header.add_cell( close_icon )
        td.add_style("text-align: right")

        return header



    def get_prev_month_wdg(my):
        prev_month = my.month - 1
        prev_year = my.year
        if prev_month < 1:
            prev_month = 12
            prev_year -= 1


        prev_month_wdg = IconButtonWdg( "Prev Month", IconWdg.LEFT )

        prev_month_wdg.add_behavior( {
            'type': "click_up",
            'cbjs_action':  '''
                var el = bvr.src_el.getParent('.spt_calendar_top');
                spt.panel.refresh(el, {year: '%s', month: '%s', is_refresh: 'true', 
                        'search_key': '%s', 'top_id': '%s',
                    'col_name': '%s'}, false);
            ''' % (prev_year, prev_month, my.search_key, my.top_id, my.col_name )
        })
        return prev_month_wdg


    def get_next_month_wdg(my):
        next_month = my.month + 1
        next_year = my.year
        if next_month > 12:
            next_month = 1
            next_year += 1


        next_month_wdg = IconButtonWdg( "Next Month", IconWdg.RIGHT )

        next_month_wdg.add_behavior( {
            'type': "click_up",
            'cbjs_action':  '''
                var el = bvr.src_el.getParent('.spt_calendar_top');
                spt.panel.refresh(el, {year: '%s', month: '%s', is_refresh: 'true', 
                        'search_key': '%s', 'top_id': '%s',
                    'col_name': '%s'}, false);
            ''' % (next_year, next_month, my.search_key, my.top_id, my.col_name )
        })
        return next_month_wdg
       

    def get_today_icon(my):

        today_icon = IconWdg("Today", IconWdg.TODAY)
        today_icon.add_class('hand')

        # button to set calendar input to today's date ...
        today_icon.add_behavior( {
            "type": "click_up",
            "cbjs_action": '''
                var top = spt.get_parent(bvr.src_el, '.spt_calendar_top');
                spt.hide( top );
                var date_obj = new Date();
                var value = date_obj.getFullYear() + '-' + spt.zero_pad(date_obj.getMonth()+1, 2) +
                            '-' + spt.zero_pad(date_obj.getDate(), 2);
                var input_top = bvr.src_el.getParent('.calendar_input_top');
                
                if (input_top) {
                    var el = input_top.getElement('.spt_calendar_input');
                    el.value = value;

                     
                    var layout = bvr.src_el.getParent(".spt_layout");
                    var version = layout ? layout.getAttribute("spt_version"): 1;
                    if (version == "2") {
                        spt.table.set_layout(layout);
                        spt.table.accept_edit(input_top, value);
                    }
                    else {
                        // A CellEditWdg containing a Calendar widget will be a detached PUW, so use 'spt.get_parent()'
                        var td = spt.get_parent( el, '.spt_table_td' );
                        if( td ) {
                            // don't set the td 'spt_input_value' here! Not necessary and might be more complex widget!
                            el.spt_text_edit_wdg.accept_edit = true;
                            spt.dg_table.edit_cell_cbk(el, spt.kbd.special_keys_map.ENTER);
                        } else {
                            spt.validation.direct_input_element_check( el );
                            el.spt_text_edit_wdg.accept_edit = true;
                        }
                    }
                }
            '''
        } )
        return today_icon
        

    def get_close_icon(my):

        close_icon = IconWdg("Close", IconWdg.POPUP_WIN_CLOSE, right_margin='0px')
        close_icon.add_class('hand')

        # button to close calendar popup ...
        close_icon.add_behavior( {
            "type": "click_up",
            "cbjs_action": '''
                var top = spt.get_parent(bvr.src_el, '.spt_calendar_top');
                spt.hide( top );
                var el = bvr.src_el.getParent('.calendar_input_top').getElement('.spt_calendar_input');

                // A CellEditWdg containing a Calendar widget will be a detached PUW, so use 'spt.get_parent()'
                var td = spt.get_parent( el, '.spt_table_td' );
                if( td ) {
                    el.spt_text_edit_wdg.accept_edit = true;
                    spt.dg_table.edit_cell_cbk(el, spt.kbd.special_keys_map.ENTER);
                } else {
                    el.spt_text_edit_wdg.accept_edit = true;
                }
            '''
        } )

        return close_icon




    def get_week_left_wdg(my, week):
        return None

    def get_week_right_wdg(my, week):
        return None


    def get_day_header_wdg(my, day):
        if my.size:
            weekday = my.WEEKDAYS[day.weekday()][0:my.size]
        else:
            weekday = my.WEEKDAYS[day.weekday()]

        div = DivWdg()
        div.add_style("border-bottom: solid 1px %s" % div.get_color("border"))

        div.add_style("font-weight: bold")
        div.add( weekday )
        return div


    def get_day_wdg(my, month, day):
        div = DivWdg()
        div.add_class("SPT_DAY_CBK")
        div.add( day.day )
        div.add_style("text-align: center")
        div.add_class("hand")

        div.add_style("padding: 3px 6px 3px 6px")

        # NOTE: generally, this is not used, but should be done with relay
        # behaviors using "spt_calendar_day"
        for cbk in my.day_cbks:
            behavior = {
                'type': 'click_up',
                'cbjs_action': cbk, 
                'search_key' : my.search_key,
                'top_id' : my.top_id,
                'col_name': my.col_name
            }
            div.add_behavior(behavior)


        div.add_class("spt_calendar_day")
        


        today = datetime.today()
        if today.year == day.year and today.month == day.month and today.day == day.day:
            div.add_color("background", "background", [-10, -10, 20])


        # put a different color for days that are not in the current month
        if day.month != month: 
            div.add_style("color: #0f0")

        # store date like the database does YYYY-MM-DD
        date_str = "%04d-%02d-%02d" % (day.year, day.month, day.day)
        div.add_attr('spt_date', date_str)
        div.add_class('spt_date_day')



        # TODO: use addClass mootools!
        color1 = div.get_color("background")
        color2 = div.get_color("background", -20)

        div.add_event("onmouseover", "$(this).setStyle('background','%s')" % color2)
        div.add_event("onmouseout", "$(this).setStyle('background','%s')" % color1)

        return div
        


class CalendarInputWdg(BaseInputWdg):

    ARGS_KEYS = {

         "first_day_of_week": {
            'description': "Integer representing first day of the week (6=Sunday, 0=Monday)",
            'type': 'SelectWdg',
            'labels': 'Sunday|Monday',
            'values': '6|0',
            'empty': 'true',
            'order': 0,
            'category': 'Display'
        },
        "show_activator": {
            'description': "Determines whether or not to show the default activator for this widget",
            'type': 'SelectWdg',
            'values': 'true|false',
            'empty': 'true',
            'order': 1,
            'category': 'internal'
        },
         "read_only": {
            'description': "Sets the widget to be read only. In this case, the calendar will not show up and only a text box with the date value will be visible.",
            'type': 'SelectWdg',
            'values': 'true|false',
            'empty': 'true',
            'order': 1,
            'category': 'Display'
        },
        'default': {
        'description': 'The default selection value in an edit form. Can be a TEL variable.',
        'type': 'TextWdg',
        'category': 'Display',
        'order': 2,
        },
        'search_key': {
        'description': 'The search key from which to extract data from',
        'type': 'TextWg',
        'category': 'internal'
        }

 
    }

    def init(my):
        my.top = DivWdg()
        my.value = ''
        my.cbjs_validation = ''
        my.validation_warning = ''

        my.day_cbks = []


    def get_top(my):
        return my.top



    def add_day_cbk(my, cbk):
        my.day_cbks.append(cbk)


    def set_validation(my, validation, validation_warning):
        my.cbjs_validation = validation
        my.validation_warning = validation_warning


    def get_display(my):
        # should not have float by default
        #my.top.add_style("float: left")
        my.top.add_class("calendar_input_top")
        # FIXME: start week with Sunday ... this is specific to MMS.  May
        # want to globalize in config for all calendars?
        first_day_of_week = my.get_option('first_day_of_week')
        if first_day_of_week == '':
            first_day_of_week = 6


        #state = my.get_state()
        #state['calendar'] = my.get_value()

	from tactic.ui.panel import EditWdg
        show_activator = my.get_option('show_activator')

        if show_activator in [True, 'true']:
            show_activator = True
        else:
            show_activator = False
        
        if isinstance(my.parent_wdg, EditWdg):
            if show_activator != 'false':
                show_activator = True
            else:
                show_activator = False



        show_calendar = my.get_option('show_calendar')
        if show_calendar in [True, 'true']:
            show_calendar = True
        else:
            show_calendar = False


        if show_activator:
            icon = IconWdg("Calendar", IconWdg.DATE)
            icon.add_class('hand')

            icon_div = DivWdg(icon)
            icon_div.add_class("spt_cal_input_show_cal_btn")  # tag this button so we can find it to hide/show
            icon_div.add_class("APP_CLICK_OFF_TOP_EL")
            icon_div.add_style("float: left")
            my.top.add(icon_div)

            icon.add_behavior( {
                "type": "click_up",
                "cbjs_action": '''
                    var el = bvr.src_el.getParent('.calendar_input_top').getElement('.spt_calendar_top');
                    spt.simple_display_toggle(el);
                '''
            } )
            clear_icon = IconWdg("Clear", IconWdg.CLOSE_INACTIVE)
            clear_icon.add_class('hand')
            clear_icon.add_style("position: absolute")
            clear_icon.add_style("padding: 2px 3px")
            
            
            clear_icon.add_behavior( {
                'type': 'click_up',
                'cbjs_action': '''
                var el = bvr.src_el.getParent('.calendar_input_top').getElement('.spt_calendar_input');
                el.value = '';
                '''
            } )
            clear_icon = FloatDivWdg(clear_icon)
            
            
            # button to set calendar input to today's date ...
            
        name = my.get_input_name()

        from tactic.ui.input import TextInputWdg
        input = TextInputWdg( name=name )

        show_time = my.get_option("show_time") in [True, 'true']
        if show_time:
            input.add_style("width: 120px")
            my.top.add_attr("show_time", "true")
        else:
            input.add_style("width: 90px")
            my.top.add_attr("show_time", "false")
        input.add_style("text-left: center")

        my.top.add(FloatDivWdg(input))

        read_only = my.get_option('read_only')
        if read_only != 'false':
            read_only = 'true'
            # This is needed because of lack of support for behaviors
            text = input.get_text()
            text.add_class("spt_calendar_input")
            text.set_option("read_only", read_only)
            text.add_event('onfocus', '''var el = $(this).getParent('.calendar_input_top').getElement('.spt_calendar_top'); spt.show(el);spt.body.add_focus_element(el)''')
            text.add_event('onclick', '''event.stopPropagation();''')
            # TODO: this onblur is nice because it hides the calendar,
            # but it stops the input from functioning
            #input.add_event('onblur', '''var el = $(this).getParent('.calendar_input_top').getElement('.spt_calendar_top'); spt.hide(el);''')

            # TODO: focus behavior not supported yet
            #input.add_behavior( {
            #'type': 'focus',
            #'cbjs_action': '''
            #var el = bvr.src_el.getParent('.calendar_input_top').getElement('.spt_calendar_top');
            #spt.show(el);'''
            #} )

        current = my.get_current_sobject()
        # has to use my.value instead of my.get_value() 
        # to avoid a display bug with multiple inputs
        default = my.get_option("default")
        if not my.value and default and current.is_insert():
            default_value = NamingUtil.eval_template(default)
            my.value = default_value

        if my.value:
            input.set_value(my.value)
        if current and not current.is_insert():
            db_date = current.get_value(my.get_name(), no_exception=True)
            if db_date:
                try:
                    value = parser.parse(db_date)
                except:
                    value = datetime.now()
                value = value.strftime("%Y/%m/%d %H:%M")
                input.set_value(value)

        kbd_bvr = {
            'type': 'keyboard',
            'kbd_handler_name': 'CalendarInputKeyboard',
        }
        input.add_behavior( kbd_bvr )


        if my.cbjs_validation:
            if my.validation_warning:
                v_warning = my.validation_warning
            else:
                v_warning = "Date entry is not valid"
            from tactic.ui.app import ValidationUtil
            v_util = ValidationUtil( direct_cbjs=my.cbjs_validation, warning=v_warning )
            v_bvr = v_util.get_validation_bvr()
            if v_bvr:
                input.add_behavior( v_bvr )
                input.add_behavior( v_util.get_input_onchange_bvr() )




        cbks = []

        if show_activator:
            my.top.add(clear_icon)

            day_cbk='''
            var top = spt.get_parent(bvr.src_el, '.spt_calendar_top');
            spt.hide( top );

            var value = bvr.src_el.getAttribute('spt_date');
            var el = bvr.src_el.getParent('.calendar_input_top').getElement('.spt_calendar_input');
            el.value = value;

            var layout = bvr.src_el.getParent(".spt_layout");
            var version = layout ? layout.getAttribute("spt_version"): 1;
            if (version == "2") {
                spt.table.set_layout(layout);
                spt.table.accept_edit(top, value);
            } else {
                // A CellEditWdg containing a Calendar widget will be a detached PUW, so use 'spt.get_parent()'
                var td = spt.get_parent( el, '.spt_table_td' );
                if( td ) {
                    // don't set the td 'spt_input_value' here! Not necessary and might be more complex widget!
                    el.spt_text_edit_wdg.accept_edit = true;
                    spt.dg_table.edit_cell_cbk(el, spt.kbd.special_keys_map.ENTER);
                } else {
                    el.spt_text_edit_wdg.accept_edit = true;
                    spt.validation.direct_input_element_check( el );
                }
            }
            '''
            cbks.append(day_cbk)

            for more_cbk in my.day_cbks:
                #calendar.add_day_cbk(cbk)
                day_cbk = "%s\n%s" % (day_cbk, more_cbk)
                cbks.append(day_cbk)

            #calendar = CalendarWdg( name=name, first_day_of_week=first_day_of_week,day_cbk=day_cbk)
            calendar = CalendarWdg( name=name, first_day_of_week=first_day_of_week)

        else:
            
            day_cbk='''
            var top = spt.get_parent(bvr.src_el, '.spt_calendar_top');

            var value = bvr.src_el.getAttribute('spt_date');
            var el = bvr.src_el.getParent('.calendar_input_top').getElement('.spt_calendar_input');
            el.value = value;

            var input_top = spt.get_parent(bvr.src_el, '.calendar_input_top');
            var show_time = input_top.getAttribute("show_time");
            if (show_time == 'true') {
                var class_name = 'tactic.ui.widget.CalendarTimeWdg';
                var kwargs = {
                    date: value
                };
                spt.panel.load(top, class_name, kwargs);
            }
            else {


            spt.hide( top );

            // A CellEditWdg containing a Calendar widget will be a detached PUW, so use 'spt.get_parent()'

            var layout = bvr.src_el.getParent(".spt_layout");
            var version = layout ? layout.getAttribute("spt_version"): 1;
            if (version == "2") {
                spt.table.set_layout(layout);
                spt.table.accept_edit(top, value);
            }
            else {
                var td = spt.get_parent( el, '.spt_table_td' );
                if( td ) {
                    // don't set the td 'spt_input_value' here! Not necessary and might be more complex widget!
                    el.spt_text_edit_wdg.accept_edit = true;
                    spt.dg_table.edit_cell_cbk(el, spt.kbd.special_keys_map.ENTER);
                } else {
                    el.spt_text_edit_wdg.accept_edit = true;
                    spt.validation.direct_input_element_check( el );
                }
            }

            }
            '''
            cbks.append(day_cbk)
            for more_cbk in my.day_cbks:
                day_cbk = "%s\n%s" % (day_cbk, more_cbk)
                cbks.append(day_cbk)

            #calendar = CalendarWdg( name=name, first_day_of_week=first_day_of_week,day_cbk=day_cbk)
            calendar = CalendarWdg( name=name, first_day_of_week=first_day_of_week)

        # add all of the callbacks to the top widget
        for day_cbk in cbks:
            my.top.add_relay_behavior( {
                'type': 'click',
                'bvr_match_class': 'spt_calendar_day',
                'cbjs_action': day_cbk
            } )



        if not show_calendar:
            calendar.add_style("display: none")

        calendar.add_style("position: absolute")
        my.top.add(calendar)

        return my.top



class CalendarMonthWdg(BaseRefreshWdg):
    MONTHS = ['January','Febuary','March','April','May','June','July','August','September','October','November','December']

    def get_display(my):

        cur_year = my.kwargs.get("year")
        cur_month = my.kwargs.get("month")
        name = my.kwargs.get('name')


        if not cur_year:
            now = datetime.now()
            cur_year = now.year
            cur_month = now.month

        top = my.top
        top.add_style("width: 180px")
        top.add_style("height: 150px")
        top.add_class("spt_month_top")


        table = Table()
        top.add(table)
        table.add_style("width: 100%")
        table.add_border()
        table.add_row()
        prev_year_wdg = IconButtonWdg( "Prev Year", IconWdg.LEFT )
        table.add_cell(prev_year_wdg)
        prev_year_wdg.add_behavior( {
            'type': 'click_up',
            'cbjs_action': '''
            var top = bvr.src_el.getParent(".spt_month_top");
            var year_el = top.getElement(".spt_year");
            year_el.innerHTML = parseInt(year_el.innerHTML) - 1;
            '''
        } )

        year_div = DivWdg()
        year_div.add_class("spt_year")
        table.add_cell(year_div)

        next_year_wdg = IconButtonWdg( "Prev Year", IconWdg.RIGHT )
        td = table.add_cell(next_year_wdg)
        td.add_style("text-align: right")
        next_year_wdg.add_behavior( {
            'type': 'click_up',
            'cbjs_action': '''
            var top = bvr.src_el.getParent(".spt_month_top");
            var year_el = top.getElement(".spt_year");
            year_el.innerHTML = parseInt(year_el.innerHTML) + 1;
            '''
        } )



        year_div.add(cur_year)
        year_div.add_style("text-align: center")
        year_div.add_style("padding: 3px")
        year_div.add_style("font-weight: bold")


        top.add_relay_behavior( {
            'type': 'mouseover',
            'bvr_match_class': 'spt_date',
            'cbjs_action': '''
            bvr.src_el.setStyle("border", "solid 1px blue");
            '''
        } )
        top.add_relay_behavior( {
            'type': 'mouseleave',
            'bvr_match_class': 'spt_date',
            'cbjs_action': '''
            bvr.src_el.setStyle("border", "solid 1px white");
            '''
        } )
        top.add_relay_behavior( {
            'type': 'click',
            'name': name,
            'bvr_match_class': 'spt_date',
            'cbjs_action': '''
            var month = bvr.src_el.getAttribute("spt_month");
            var top = bvr.src_el.getParent(".spt_month_top");
            var year_el = top.getElement(".spt_year");
            var year = parseInt(year_el.innerHTML);

            var class_name = 'tactic.ui.widget.CalendarWdg';
            var kwargs = {
                col_name: bvr.name,
                year: year,
                month: month,
                is_refresh: true
            };
            var top = spt.get_parent(bvr.src_el, '.spt_calendar_top');
            spt.panel.load(top, class_name, kwargs);
            
            '''
        } )




        for i, month in enumerate(my.MONTHS):
            month_div = DivWdg()
            month_div.add(month[:3])
            month_div.add_class("spt_date")
            month_div.add_attr("spt_month", i+1)
            month_div.add_style("float: left")
            month_div.add_style("padding: 9px 11px")
            month_div.add_style("border: solid 1px white")
            top.add(month_div)

            if i+1 == cur_month:
                month_div.add_color("background", "background3")

        return top



class CalendarTimeWdg(BaseRefreshWdg):

    def get_display(my):
        top = my.top
        top.add_style("width: 180px")
        top.add_style("height: 150px")

        hours = my.kwargs.get("hours")
        if not hours:
            hours = 0
        minutes = my.kwargs.get("minutes")
        if not minutes:
            minutes = 0

        #input = TextWdg("date")
        #top.add(input)
        #input.add_class("spt_calendar_input")

        date = my.kwargs.get("date")
        if date:
            try:
                date = parser.parse(date)
            except:
                date = datetime.now()
        else:
            date = datetime.now()

        title_div = DivWdg()
        top.add(title_div)
        title_div.add(date.strftime("%a - %b %d, %Y"))

        title_div.add_behavior( {
            'type': 'click_up',
            'cbjs_action': '''
            var top = spt.get_parent(bvr.src_el, '.spt_calendar_top');

            var value = bvr.src_el.getAttribute('spt_date');
            //var el = bvr.src_el.getParent('.spt_calendar_top').getElement('.spt_calendar_input');
            var el = bvr.src_el.getParent('.calendar_input_top').getElement('.spt_calendar_input');
            el.value = value;

            var class_name = 'tactic.ui.widget.CalendarWdg';
            var kwargs = {
                date: value,
                is_refresh: true,
            };
            spt.panel.load(top, class_name);
            ''' } )




        name = my.kwargs.get("name")
       
        top.add_behavior( {
            'type': 'load',
            'cbjs_action': '''
            spt.time_input = {};
            spt.time_input.drag_start_x = null;
            spt.time_input.drag_start_value = null;

            spt.time_input.drag_setup = function(evt, bvr, mouse_411) {
                spt.time_input.drag_start_x = mouse_411.curr_x;
                var src_el = spt.behavior.get_bvr_src( bvr );
                if (!src_el.value) {
                    src_el.value = 0;
                }
                spt.time_input.drag_start_value = src_el.value;
                src_el.focus();
            }
            spt.time_input.drag_motion = function(evt, bvr, mouse_411) {
                var start_value = spt.time_input.drag_start_value; 
                var dx = mouse_411.curr_x - spt.time_input.drag_start_x;

                var src_el = spt.behavior.get_bvr_src( bvr );
                var unit = src_el.getAttribute("spt_unit");
                var max = 23;
                var start = "12";
                var interval = 10;
                if (unit == 'minute') {
                    max = 59;
                    start = "00"
                    interval = 5;
                }

                var diff = parseInt(dx / interval);

                var src_el = spt.behavior.get_bvr_src( bvr );
                try {
                    var value = parseInt(start_value) + diff;

                    if (value >= max) {
                        value = 0;
                    }
                    else if (value <= 0) {
                        value = max;
                    }


                    src_el.value = spt.zero_pad(value, 2);
                }
                catch(e) {
                    src_el.value = start;
                }
            }
            '''
        } )

        top.add_behavior( {
            'type': 'smart_drag',
            'bvr_match_class': 'spt_time_element',
            "cbjs_setup": 'spt.time_input.drag_setup( evt, bvr, mouse_411 );',
            "cbjs_motion": 'spt.time_input.drag_motion( evt, bvr, mouse_411 );'
        } )

        table = Table()
        top.add(table)
        table.add_row()
        table.add_style("font-size: 20px")
        table.add_style("font-weight: bold")
        table.add_style("margin: 20px auto")

        hours_input = TextWdg("%s|hours" % name)
        hours_input.add_class("spt_time_element")
        hours_input.add_attr("spt_unit", "hour")
        hours_input.add_style("height: 50px")
        hours_input.add_style("width: 50px")
        hours_input.add_style("font-size: 20px")
        hours_input.add_style("text-align: center")
        hours_input.set_value("%0.2d" % hours)

        minutes_input = TextWdg("%s|minutes" % name)
        minutes_input.add_class("spt_time_element")
        minutes_input.add_attr("spt_unit", "minute")
        minutes_input.add_style("height: 50px")
        minutes_input.add_style("width: 50px")
        minutes_input.add_style("text-align: center")
        minutes_input.add_style("font-size: 20px")
        minutes_input.set_value("%0.2d" % minutes)

        left = table.add_cell()
        left.add(hours_input)

        middle = table.add_cell(":")
        middle.add_style("width: 15px")
        middle.add_style("text-align: center")

        right = table.add_cell()
        right.add(minutes_input)

        ok_div = DivWdg()
        top.add(ok_div)
        ok_div.add_style("float: right")
        from tactic.ui.widget import ActionButtonWdg
        button = ActionButtonWdg(title="Accept")
        ok_div.add(button)

        date_str = date.strftime("%Y/%m/%d")
        ok_div.add_behavior( {
            'type': 'click_up',
            'date': date_str,
            'cbjs_action': '''
            var top = bvr.src_el.getParent('.spt_calendar_top');
            var time_els = top.getElements('.spt_time_element');
            var hour = time_els[0].value;
            var mins = time_els[1].value;

            var value = bvr.date + " " + hour + ":" + mins;
            spt.hide( top );

            var input_top = bvr.src_el.getParent('.calendar_input_top');
            var el = input_top.getElement('.spt_calendar_input');
            el.value = value;


            // only support version 2 of table layout
            var layout = bvr.src_el.getParent(".spt_layout");
            if (layout) {
                spt.table.set_layout(layout);
                spt.table.accept_edit(top, value);
            } 

            '''
        } )



        top.add("<br clear='all'/>")

        return top

 




class TimeInputWdg(BaseInputWdg):

    def get_display(my):

 
        div = FloatDivWdg()
        div.add_class("spt_time_top")


        my.time_cbk = my.get_option('time_cbk')
        show_activator = my.get_option('show_activator')
        if show_activator == "" or show_activator == True:
            show_activator = True
        else:
            show_activator = False

        is_embedded = my.get_option("is_embedded")
        if is_embedded in ['true', True]:
            is_embedded = True
        else:
            is_embedded = False
            show_activator = False



        show_confirm = my.get_option('show_confirm')
        if show_confirm == "":
            # EditWdg doesn't have the confirm OK button
            from tactic.ui.panel import EditWdg
            if isinstance(my.get_parent_wdg(), EditWdg):
                show_confirm = False
            else:
                # for used in TableLayoutWdg or FastTableLayoutWdg
                show_confirm = True
            
        elif show_confirm in ['false', False]:
            show_confirm = False
        else:
            show_confirm = True

        # there is an assumption it is used beside a CalendarWdg
        if show_activator:
            icon = IconWdg("Date", IconWdg.DATE)
            div.add(icon)

            icon.add_behavior( {
                "type": "click",
                "cbjs_action": '''var el = bvr.src_el.getParent('.calendar_input_top').getElement('.spt_calendar_top');
                    spt.simple_display_toggle(el);
                '''
            } )

        name = my.get_input_name()

        interval = my.get_option("interval")
        if interval:
            interval = int(interval)
        else:
            interval = 10


        # set a default time
        value = my.get_value()
        is_expr = False
        if value:
            if value.find('$') != -1:
                is_expr = True
            else:
                date = parser.parse(value)
                value = str(date.time())

        if not value or is_expr:
            value = "12:00"

        parts = value.split(":")
        
        # keep them as strings to match the select string values
        hour = parts[0]
        min = parts[1]


        hour_name = my.get_input_name("%s|hour" % my.name)
        minute_name = my.get_input_name("%s|minute" % my.name)
        hr_select = SelectWdg(hour_name)
        hr_select.add_class('spt_time_hour')
        hr_select.set_value(hour)
        values =  [str(x).zfill(2) for x in xrange(0,24)]

        hr_select.set_option('values', values)
        hr_select.set_option('labels', values)
        div.add(hr_select)

        div.add(" : ")

        select = SelectWdg(minute_name)
        select.add_class('spt_time_minute')
        select.set_value(min)
        min_range = range(0, 60, interval)
        values_list = [str(x).zfill(2) for x in min_range]
        select.set_option('values', values_list )
        select.set_option('labels', values_list )
        div.add(select)
      
        show_confirm = False
        if show_confirm:
            ok = ProdIconButtonWdg('ok')
            ok.add_color('background','background2')
            ok.add_behavior({'type': 'click_up',
                'cbjs_action' : my.time_cbk})
            div.add(ok)
        else:
            # add to the hr/ minute select
            hr_select.add_behavior({'type': 'change',
                            'cbjs_action': my.time_cbk})
            select.add_behavior({'type': 'change',
                            'cbjs_action': my.time_cbk})


        if not is_embedded:
            div.add_style("padding: 20px")
            div.add_border()
            div.add_color("background", "background")

            # This is special code for table layout
            button = IconButtonWdg("OK", IconWdg.EDIT)
            div.add(button)
            button.add_behavior( {
                'type': 'click_up',
                'prefix': my.get_name(),
                'cbjs_action': '''
                var top_el = bvr.src_el.getParent(".spt_time_top");
                var values = spt.api.Utility.get_input_values(top_el);
                value_wdg = bvr.src_el;
                var hour = values[bvr.prefix+"|hour"];
                var minute = values[bvr.prefix+"|minute"];
                value_wdg.value = hour+":"+minute;
                spt.table.accept_edit(top_el, value_wdg.value, true);
                '''
            } )

        div.add_class("spt_no_alter")

        return div


__all__.append("TimeDatabaseAction")
from pyasm.command import DatabaseAction
class TimeDatabaseAction(DatabaseAction):

    def execute(my):
        name = my.get_name()

        column = my.get_option("column")
        if not column:
            column = name

        value = my.get_value()
        if value:
            parts = value.split(":")
            hour = int(parts[0])
            minute = int(parts[1])
        else:
            hour = 12
            minute = 0

        sobject = my.sobject
        cur_value = sobject.get_value(column)
        if cur_value:
            cur_value = parser.parse(value)
        else:
            cur_value = datetime.now()


        column_info = sobject.get_column_info(column)
        if column_info.get("data_type") in ['timestamp']:
            cur_value = cur_value.replace(hour=hour, minute=minute)
        else:
            cur_value = value

        sobject.set_value(column, cur_value)

        return name





class CalendarTimeInputWdg(CalendarInputWdg):
    '''CalendarInput + TimeputWdg for datetime type column'''
    ARGS_KEYS = CalendarInputWdg.ARGS_KEYS.copy()
    ARGS_KEYS.update({
       "interval": {
            'description': "Interval for minutes, default to 10",
            'type': 'SelectWdg',
            'values': '1|5|10|20|30',
            'empty': 'true',
            'order': 2,
            'category': 'Display'
        }
    })



    def get_display(my):
        #TODO: if this widget is aware of its parent widget's layout version
        # we can target the proper javascript for the day_cbk and time_cbk

        my.top.add_class("calendar_input_top")

        # NOTE: start week with Sunday May want to globalize in config
        # for all calendars?
        first_day_of_week = my.get_option('first_day_of_week')
        if first_day_of_week == '':
            first_day_of_week = 6


        show_activator = my.get_option('show_activator')
        show_text = False

        if show_activator == "" or show_activator in [True,'true']:
            show_activator = True
            show_text = True
        else:
            show_activator = False
       

        if show_activator:
            show_text = my.get_option('show_text')
            if show_text in [False, 'false']:
                show_text = False
            else:
                show_text = True
        
            icon = IconWdg("Calendar", IconWdg.DATE)
            icon.add_class('hand')

            icon_div = DivWdg(icon)
            icon_div.add_class("spt_cal_input_show_cal_btn")  # tag this button so we can find it to hide/show
            icon_div.add_class("APP_CLICK_OFF_TOP_EL")
            icon_div.add_style("float: left")
            my.top.add(icon_div)

            icon.add_behavior( {
                "type": "click_up",
                "cbjs_action": '''
                    var el = bvr.src_el.getParent('.calendar_input_top').getElement('.spt_calendar_top');
                    spt.simple_display_toggle(el);
                '''
            } )
           
            today_icon = IconWdg("Today", IconWdg.ADD)
            today_icon.add_class('hand')

            # button to set calendar input to today's date ...
            today_icon.add_behavior( {
                "type": "click_up",
                "cbjs_action": '''
                    var date_obj = new Date();
                    var value = date_obj.getFullYear() + '-' + spt.zero_pad(date_obj.getMonth()+1, 2) +
                                '-' + spt.zero_pad(date_obj.getDate(), 2);
                    var el = bvr.src_el.getParent('.calendar_input_top').getElement('.spt_calendar_input');
                    el.value = value;

                    // A CellEditWdg containing a Calendar widget will be a detached PUW, so use 'spt.get_parent()'
                    var td = spt.get_parent( el, '.spt_table_td' );
                    if( td ) {
                        // don't set the td 'spt_input_value' here! Not necessary and might be more complex widget!
                        el.spt_text_edit_wdg.accept_edit = true;
                        spt.dg_table.edit_cell_cbk(el, spt.kbd.special_keys_map.ENTER);
                    } else {
                        el.spt_text_edit_wdg.accept_edit = true;
                        spt.validation.direct_input_element_check( el );
                    }
                '''
            } )

            today_icon = FloatDivWdg(today_icon)
            #today_icon.add_style("float: right")
            today_icon.add_class("spt_cal_input_today_btn")  # tag this button so we can find it to hide/show
          
            
        name = my.get_input_name()
        input = TextWdg( name=name )
        input.add_style("display: none")
        display_input = TextWdg( name="%s_display" % name )
        display_input.add_style("width: 70px")

        read_only = my.get_option('read_only')
        if read_only not in ['false', False]:
            read_only = 'true'
            #input.set_option("read_only", read_only)
            display_input.set_option("read_only", read_only)



      
        if my.value:
            input.set_value(my.value)
            display_input.set_value(my.value)
        current = my.get_current_sobject()
        if current and not current.is_insert():
            db_date = current.get_value(my.get_name())
            if db_date:
                date = Date(db=db_date)
                input.set_value(date.get_db_time(), set_form_value=False)
                display_input.set_value(date.get_db_time(), set_form_value=False)

        kbd_bvr = {
            'type': 'keyboard',
            'kbd_handler_name': 'CalendarInputKeyboard',
        }
        # this only matters with old table
        input.add_behavior( kbd_bvr )


        if my.cbjs_validation:
            if my.validation_warning:
                v_warning = my.validation_warning
            else:
                v_warning = "Date entry is not valid"
            from tactic.ui.app import ValidationUtil
            v_util = ValidationUtil( direct_cbjs=my.cbjs_validation, warning=v_warning )
            v_bvr = v_util.get_validation_bvr()
            if v_bvr:
                input.add_behavior( v_bvr )
                input.add_behavior( v_util.get_input_onchange_bvr() )
        
        if not show_text:
            input.add_style('display: none')
            display_input.add_style('display: none')

        input.add_class("spt_calendar_input")
        display_input.add_class("spt_calendar_display")
        input_div = FloatDivWdg()
        input_div.add(input)
        input_div.add(display_input)

        input_div.add_style('margin-right: 5px')
        my.top.add(input_div)
        

        # calenar portion
        if show_activator:
            my.top.add(SpanWdg('&nbsp;',css='small'))
            #my.top.add(today_icon)

            day_cbk='''
            var top = spt.get_parent(bvr.src_el, '.spt_calendar_top');
            spt.hide( top );

            var input_top = bvr.src_el.getParent('.calendar_input_top');
            var value = bvr.src_el.getAttribute('spt_date');
            var el = bvr.src_el.getParent('.calendar_input_top').getElement('.spt_calendar_display');
            el.value = value;
            var el = bvr.src_el.getParent('.calendar_input_top').getElement('.spt_calendar_input');
            el.value = value;
    
            var time_top = input_top.getElement('.spt_time_top');
            var time_values = spt.api.Utility.get_input_values(time_top);
            if ( value && value.test(/^\d\d\d\d-\d\d-\d\d( .*)?/) ) {
                 var parts = value.split(' ');
                 var new_value = parts[0] + ' ' + spt.zero_pad(time_values['%s|hour'], 2) + ':' + spt.zero_pad(time_values['%s|minute'], 2);
                 el.value = new_value;
            }
           
            el.spt_text_edit_wdg.accept_edit = true;
            spt.validation.direct_input_element_check( el );
            '''%(name, name)
            for more_cbk in my.day_cbks:
                #calendar.add_day_cbk(cbk)
                day_cbk = "%s\n%s" % (day_cbk, more_cbk)

            calendar = CalendarWdg( name=name, first_day_of_week=first_day_of_week,day_cbk=day_cbk)

        else:
            day_cbk='''
            var top = spt.get_parent(bvr.src_el, '.spt_calendar_top');
            spt.hide( top );
            var input_top = bvr.src_el.getParent('.calendar_input_top');
            var value = bvr.src_el.getAttribute('spt_date');
            var el = input_top.getElement('.spt_calendar_input');
            el.value = value;
            
            var time_top = input_top.getElement('.spt_time_top');
            var time_values = spt.api.Utility.get_input_values(time_top);
            if ( value && value.test(/^\d\d\d\d-\d\d-\d\d( .*)?/) ) {
                 var parts = value.split(' ');
                 var new_value = parts[0] + ' ' + spt.zero_pad(time_values['%s|hour'], 2) + ':' + spt.zero_pad(time_values['%s|minute'], 2);
                 el.value = new_value;
            }

            var layout = bvr.src_el.getParent(".spt_layout");
            var version = layout ? layout.getAttribute("spt_version"): 1;
            if (version == "2") {
                spt.table.set_layout(layout);
                spt.table.accept_edit(input_top, new_value);
            }
            else {
                // A CellEditWdg containing a Calendar widget will be a detached PUW, so use 'spt.get_parent()'
                var td = spt.get_parent( el, '.spt_table_td' );
                if( td ) {
                    // don't set the td 'spt_input_value' here! Not necessary and might be more complex widget!
                    //el.spt_text_edit_wdg.accept_edit = true;
                    //spt.dg_table.edit_cell_cbk(el, spt.kbd.special_keys_map.ENTER);
                    // Need to set it here since it is a Calendar time combo. and don't want to return to home cell yet
                    td.setAttribute('spt_input_value', el.value);
                } else {
                    el.spt_text_edit_wdg.accept_edit = true;
                    //spt.validation.direct_input_element_check( el );
                }
            }
            '''%(name, name)
            for more_cbk in my.day_cbks:
                #calendar.add_day_cbk(cbk)
                day_cbk = "%s\n%s" % (day_cbk, more_cbk)


            calendar = CalendarWdg( name=name, first_day_of_week=first_day_of_week,day_cbk=day_cbk)

        #for cbk in my.day_cbks:
        #    calendar.add_day_cbk(cbk)

        #if show_activator:
        calendar.add_style("display: none")

        calendar.add_style("position: absolute")
        my.top.add(calendar)

        elem_dict = {'name': name}
        
        # time portion
        time_cbk='''
            var top = spt.get_parent(bvr.src_el, '.spt_calendar_top');

            var input_top = bvr.src_el.getParent('.calendar_input_top');
            var el = input_top.getElement('.spt_calendar_input');

            var Utility = spt.api.Utility;

            var layout = bvr.src_el.getParent(".spt_layout");
            var version = layout ? layout.getAttribute("spt_version"): 1;

            //var value = bvr.src_el.getAttribute('spt_date');
            var value = el.value;
            if (version == "2") {
                // spt.table.set_layout(layout);
                // spt.table.accept_edit(input_top, value);

                var time_top = input_top.getElement('.spt_time_top');
                var time_values = Utility.get_input_values(time_top);
                if (!value) {
                   var date_obj = new Date();
                   value = date_obj.getFullYear() + '-' + spt.zero_pad(date_obj.getMonth()+1, 2) +
                            '-' + spt.zero_pad(date_obj.getDate(), 2);
                }
                if ( value && value.test( /^\d\d\d\d-\d\d-\d\d( .*)?/ ) ) {
                     var parts = value.split(' ');
                     var new_value = parts[0] + ' ' + spt.zero_pad(time_values['%(name)s|hour'], 2) + ':' + spt.zero_pad(time_values['%(name)s|minute'], 2);
                     var text = Utility.get_input(input_top, '%(name)s','.spt_calendar_input');
                     text.value = new_value;
                }
                spt.table.set_layout(layout);
                spt.table.accept_edit(input_top, new_value);
            }
            else {

                // A CellEditWdg containing a Calendar widget will be a detached PUW, so use 'spt.get_parent()'
                var td = spt.get_parent( el, '.spt_table_td' );
                if( td ) {

                    el.value = value;
                    //var value = td.getAttribute('spt_input_value');
                    var text = Utility.get_input(td, '%(name)s','.spt_calendar_input');
                    var time_top = td.getElement('.spt_time_top');
                    var time_values = Utility.get_input_values(time_top);
                    // auto use today's date if it's not set
                    if (!value) {
                       var date_obj = new Date();
                       value = date_obj.getFullYear() + '-' + spt.zero_pad(date_obj.getMonth()+1, 2) +
                                '-' + spt.zero_pad(date_obj.getDate(), 2);
                    }
                    if ( value && value.test( /^\d\d\d\d-\d\d-\d\d( .*)?/ ) ) {
                         var parts = value.split(' ');
                         var new_value = parts[0] + ' ' + spt.zero_pad(time_values['%(name)s|hour'], 2) + ':' + spt.zero_pad(time_values['%(name)s|minute'], 2);
                         text.value = new_value;
                    }
                   
                    el.spt_text_edit_wdg.accept_edit = true;
                    spt.dg_table.edit_cell_cbk(el, spt.kbd.special_keys_map.ENTER);
                    
                } else {
                    
                    var text = Utility.get_input(input_top, '%(name)s','.spt_calendar_input');
                    var time_top = input_top.getElement('.spt_time_top');
                    var time_values = Utility.get_input_values(time_top);
                    var value = text.value;

                    if (!value) {
                       var date_obj = new Date();
                       value = date_obj.getFullYear() + '-' + spt.zero_pad(date_obj.getMonth()+1, 2) +
                                '-' + spt.zero_pad(date_obj.getDate(), 2);
                    }
                    if ( value && value.test( /^\d\d\d\d-\d\d-\d\d( .*)?/ ) ) {
                         var parts = value.split(' ');
                         var new_value = parts[0] + ' ' + spt.zero_pad(time_values['%(name)s|hour'], 2) + ':' + spt.zero_pad(time_values['%(name)s|minute'], 2);
                         text.value = new_value;
                    }

                    el.spt_text_edit_wdg.accept_edit = true;
                    spt.validation.direct_input_element_check( el );
                }
            }
            '''%(elem_dict)
        
        time_input = TimeInputWdg(name=name)
        time_input.set_option('is_embedded', True)

        # this is for TimeInputWdg to be aware if it is being drawn in EditWdg 
        # or TableLayout
        time_input.set_parent_wdg(my.parent_wdg)
        time_input.set_option('time_cbk', time_cbk)
        time_input.set_option('show_activator', not show_activator)
        time_input.set_option('show_confirm', my.get_option('show_confirm'))
        interval = my.get_option('interval')
        if interval:
            time_input.set_option('interval', interval)
        my.top.add(time_input)

        return my.top


